"""
Created on 25 fevr. 2019

@author: Stephane Tonneau

"""
from oscars_mgmnt_clms.ingestion import MetadataIngester
from oscars_mgmnt_clms.utils.common import Config
from oscars_mgmnt_clms.utils.storage import FileSystemInput
from oscars_mgmnt_clms.management import CollectionDefinitions, CollectionMgmnt, ElasticsearchObject
from oscars_mgmnt_clms.remove import ProductRemoveHelper
from fire.core import FireError
import oscars_mgmnt_clms

errorMessageMandatoryConfig = "No value has been set for the {} parameter. Supply it as an argument or in the " \
                              "configuration file."


def confirm(message):
    """ asks for a confirmation, with the provided message.

    :param message: The message to print for confirmation
    :type message: a string
    :returns: True if the user has select yes.
    :rtype: a boolean
    """

    print(message + " [y/n]")
    try:
        my_input = raw_input
    except NameError:
        my_input = input
    choice = my_input().lower()
    return choice == 'y'


class Collection(object):
    """This sub-commands provides facilities to manage the definitions of EO collections.

    Each collection is described by a geojson file, that is stored within an elasticsearch
    server.  This tool allows the user to insert, delete or update such definitions.  The
    edition and the creation of the geojson file can be realized by a regular text editor.
    """

    def __init__(self):
        self._coll_def = None
        self._configuration = None
        self._init_conf = True

    def __str__(self):
        """Blank string representation of the class to avoid
        unfriendly object display
        """
        return ""

    def _get_coll_def(self):
        """ initialise a CollectionDefinitions object with the configuration file

        :returns: the object itself
        :rtype: a CollectionDefinitions
        """
        if self._configuration is None:
            self._configuration = Config()
        if self._coll_def is None:
            self._coll_def = CollectionDefinitions(self._configuration)
        return self._coll_def

    def _configure(self, path=None):
        """With a filename as argument, this directive provides a preconfigured context
        (url of the Elasticseach server, ... )for the commands that follow it.

        This command expects a filename as argument.  The pointed file shoud contains a python
        configuration file with an [elasticsearch] section that provides entries for the following keys :
        1) servers : a list of machines, referenced by their name ( the comma is used a separator )
        2) collection-index : the elastic search index to use for storing the definition of the collections.

        Example: configuration /tmp/myconf.conf list

        :param path: The name of the file containing the configuration
        :type path: a string
        :returns: the object itself
        :rtype: a Collection
        """
        self._configuration = Config(path, self._init_conf)
        return self

    def list(self, configuration=None):
        """Lists the collections supported by the system

        :param configuration: the path to a file that contains the configuration
        :type configuration: a string
        """
        self._configure(configuration)
        definitions = self._get_coll_def()
        res = definitions.list()
        for key in res:
            print(key + " : " + res[key])

    def get(self, urn, configuration=None):
        """With the urn as argument, this command retrieves the geojson description file
        associated to a collection.

        :param urn: The unique reference of the collection
        :param configuration: the path to a file that contains the configuration
        :type urn: a string
        :type configuration: a string
        """
        self._configure(configuration)
        definitions = self._get_coll_def()
        res = definitions.get_string(urn)
        if res is not None:
            print(res)
        else:
            raise Exception("No collection available with reference " + str(urn))

    def put(self, filename, schema=None, configuration=None):
        """With a file path as argument, this command creates or updates the geojson description file of a collection.

        :param filename: The path of a geojson file describing the collection
        :param schema: the path to a schema for validation
        :param configuration: the path to a file that contains the configuration
        :type filename: a string
        :type schema: a string
        :type configuration: a string
        """
        self._configure(configuration)
        definitions = self._get_coll_def()
        validator = self._configuration.collections_schema if schema is None else schema
        validator = validator if (validator is not None) and (len(validator) > 0) else None

        def fill_download_template(data):
            download_host = self._configuration.download_url_prefix
            if download_host:
                if "dataAccess" in data["hiddenInformation"]:
                    for key in data["hiddenInformation"]["dataAccess"]:
                        try:
                            data["hiddenInformation"]["dataAccess"][key]["dst"] =\
                                data["hiddenInformation"]["dataAccess"][key]["dst"].replace("{URL_PREFIX}",
                                                                                            download_host)
                        except KeyError:
                            pass
                elif "httpDataAccess" in data["hiddenInformation"]:  # remain backwards compatible
                    try:
                        data["hiddenInformation"]["httpDataAccess"]["dst"] =\
                            data["hiddenInformation"]["httpDataAccess"]["dst"].replace("{URL_PREFIX}", download_host)
                    except KeyError:
                        pass
            return data

        definitions.put_file(filename, validator, modifier=fill_download_template)

    def delete(self, urn, force=False, configuration=None):
        """With the urn as argument, this command deletes the geojson description file associated to a collection.
        The collection itself, and the associated elasticsearch index shall not be removed.

        :param urn: The unique reference of the collection
        :param force: indicates if the operation confirmation can be avoided
        :param configuration: the path to a file that contains the configuration
        :type urn: a string
        :type force: a boolean
        :type configuration: a string
        """
        self._configure(configuration)
        definitions = self._get_coll_def()
        if force or confirm("Are you sure you want to remove this collection " + str(urn) + " ? "):
            res = definitions.delete(urn)
            if res is None:
                raise Exception("No collection available with reference " + str(urn))
        else:
            print('Nothing done ...')


class Manage(object):
    """This sub-commands provides facilities to manage Elasticsearch indexes in the context
    of EO collections.

    The metadafiles of products can be ingested in specific indexes of the elasticsearch server.
    This module provides support for the management of the indexes.
    """

    def __init__(self):
        self._manager = None
        self._configuration = None
        self._init_conf = True

    def __str__(self):
        """Blank string representation of the class to avoid
        unfriendly object display
        """
        return ""

    def _get_manager(self) -> CollectionMgmnt:
        """ initialise a CollectionMgmnt object with the configuration file

        :returns: CollectionMgmnt object
        """
        if self._configuration is None:
            self._configuration = Config()
        if self._manager is None:
            self._manager = CollectionMgmnt(self._configuration)
        return self._manager

    def _configure(self, filename=None):
        """With a filename as argument, this directive provides a preconfigured context
        (url of the Elasticseach server, ... )for the commands that follow it.

        This command expects a filename as argument.  The pointed file shoud contains a python
        configuration file with an [elasticsearch] section that provides entries for the following keys :
        1) servers : a list of machines, referenced by their name ( the comma is used a separator )
        2) collection-index : the elastic search index to use for storing the definition of the collections.

        Example: configuration /tmp/myconf.conf create urn:xxx:yyy

        :param filename: The name of the file containing the configuration
        :type filename: a string
        :returns: the object itself
        :rtype: a Manage
        """
        self._configuration = Config(filename, self._init_conf)
        return self

    def create(self, index, alias=True, configuration=None):
        """With an index reference as argument, this command creates a new collection
        storage location with the referenced Elasticsearch index.

        :param index: The index of the collection
        :param alias: boolean indicating whether to move or create the alias
        :param configuration: the path to a file that contains the configuration
        :type index: a string
        :type configuration: a string
        """
        self._configure(configuration)
        manager = self._get_manager()
        manager.create(index, alias)

    def remove(self, index, force=False, configuration=None):
        """With an index reference as argument, this command deletes the collection
        storage location referenced by the Elasticsearch index.  The previously
        ingested product metadafile of the collection shall be definitively removed after
        the execution of the operation.

        :param index: The index of the collection
        :param force: indicates if the operation confirmation can be avoided
        :param configuration: the path to a file that contains the configuration
        :type index: a string
        :type force: a boolean
        :type configuration: a string
        """
        if force or confirm("Are you sure you want to remove index " + str(index) + " ? "):
            self._configure(configuration)
            manager = self._get_manager()
            manager.remove(index)
        else:
            print('Nothing done ...')

    def alias(self, alias: str, index: str, configuration=None):
        """
        Set or move the Elasticsearch alias to the specified index.

        :param alias: Elasticsearch alias
        :param index: Elasticsearch index
        :param configuration: the path to a configuration file
        """
        self._configure(configuration)
        manager = self._get_manager()
        manager.alias(alias, index)

    def _initoscars(self, index=None, configuration=None):
        """Without the necessity of any argument, this command initializes the oscars system,
        by creating the Elasticsearch index necessary to store the definition of collections.

        :param index: The index of the collection definition storage
        :param configuration: the path to a file that contains the configuration
        :type index: a string
        :type configuration: a string
        """
        self._configure(configuration)
        manager = self._get_manager()
        collections_index = self._configuration.collections_index if index is None else index
        manager.initialise(collections_index)


class Products(object):
    """This sub-commands provides facilities to ingest the metadatafiles associated to EO products.

    The metadatafiles of products can be ingested
    1) in specific storage location ( kafka, elasticsearch, filesystem, .. )
    2) using specific templates to translate their xml form to geojson files
    3) with a specific context ( product and metadatas in the same repository, or separated, ... ).
    """

    def __str__(self):
        """Blank string representation of the class to avoid
        unfriendly object display
        """
        return ""

    def __init__(self):
        self._configuration = None
        self._init_conf = True
        self._result_writer = []

    def _get_index(self, urn):
        """With the urn as argument, this command tries to retrieve the collection index.
        starting from the product urn

        :param urn: The unique reference of the product
        :type urn: a string
        :returns: the index of the associated collection
        :rtype: a string
        """
        definitions = CollectionDefinitions(self._configuration)
        listing = definitions.list()
        res = None
        for key in listing:
            if (key + ":") in urn:
                if res is None:
                    res = listing[key]
                else:
                    raise Exception("Several collections match the product urn " + str(urn))
        return res

    def get(self, urn, index, configuration=None):
        """With the urn as argument, this command retrieves the geojson description file
        associated to an index.

        :param urn: The unique reference of the product
        :param index: the index of the collection in Elasticsearch
        :param configuration: the path to a file that contains the configuration
        :type urn: a string
        :type index: a string
        :type configuration: a string
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        es_index = self._get_index(urn) if index is None else index
        es_index = self._configuration.es_index if es_index is None else es_index
        coll_object = ElasticsearchObject(self._configuration, es_index)
        res = coll_object.get_string(urn)
        if res is not None:
            print(res)
        else:
            raise Exception("No product available with reference " + str(urn))

    def put(self, filename, index, schema=None, configuration=None):
        """With a file path as argument, this command creates or updates the metadafile of a product in a
        specific elasticsearch index.

        :param filename: The path of a geojson file describing the collection
        :param index: the index of the collection in Elasticsearch
        :param schema: the path to a schema for validation
        :param configuration: the path to a file that contains the configuration
        :type filename: a string
        :type index: a string
        :type schema: a string
        :type configuration: a string
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        validator = self._configuration.products_schema if schema is None else schema
        validator = validator if (validator is not None) and (len(validator) > 0) else None
        es_index = index
        coll_object = ElasticsearchObject(self._configuration, es_index)
        json_data = coll_object.get_json(filename)
        if es_index is None:
            es_index = self._get_index(json_data['id'])
            es_index = self._configuration.es_index if es_index is None else es_index
            coll_object = ElasticsearchObject(self._configuration, es_index)
        coll_object.put(json_data, validator)

    def delete(self, urn, index, force=False, configuration=None):
        """With the urn as argument, this command deletes the geojson description file associated to a 
        given product.

        :param urn: The unique reference of the collection
        :param index: the index of the collection in Elasticsearch
        :param force: indicates if the operation confirmation can be avoided
        :param configuration: the path to a file that contains the configuration
        :type urn: a string
        :type index: a string
        :type force: a boolean
        :type configuration: a string
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        es_index = self._get_index(urn) if index is None else index
        es_index = self._configuration.es_index if es_index is None else es_index
        coll_object = ElasticsearchObject(self._configuration, es_index)
        if force or confirm("Are you sure you want to delete this product ?" + str(urn)):
            res = coll_object.delete(urn)
            if res is None:
                raise Exception("No product available with reference " + str(urn))
        else:
            print('Nothing done ...')

    def deleteWildcard(self, wildcard: str, index: str, force: bool = False, configuration: str = None):
        """
        Delete the documents where the ID matches the provided wildcard expression.

        :param wildcard: wildcard expression
        :param index: the index of the collection in Elasticsearch
        :param force: skip confirmation prompt
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        coll_obj = ElasticsearchObject(self._configuration, index)
        matches = coll_obj.listIDs(wildcard)

        if force or confirm(f"Are you sure you want to delete {len(matches)} products?"):
            for id in matches:
                res = coll_obj.delete(id)
                if res is None:
                    raise Exception(f"No product available with reference {id}")
        else:
            print("Nothing done...")

    def deleteList(self, ids: str, index: str, force: bool = False, configuration: str = None):
        """
        Delete a list of products by ID.

        :param ids: list of urn IDs of documents to delete
        :param index: the index of the collection in Elasticsearch
        :param force: skip confirmation prompt
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        coll_obj = ElasticsearchObject(self._configuration, index)

        if force or confirm(f"Are you sure you want to delete {len(ids)} products?"):
            for id in ids:
                res = coll_obj.delete(id)
                if res is None:
                    raise Exception(f"No product available with reference {id}")
        else:
            print("Nothing done...")

    def remove(self, urn: str, index: str, force: bool = False, configuration: str = None):
        """
        Mark a product as removed in Elasticsearch.
        Product status will be updated to CANCELLED and the product file information is removed.

        :param urn: product ID
        :param index: the index of the collection in Elasticsearch
        :param force: skip confirmation prompt
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        coll_obj = ElasticsearchObject(self._configuration, index)
        if force or confirm(f"Are you sure you want to remove this product? {urn}"):
            doc = coll_obj.get(urn)
            doc = ProductRemoveHelper.fix_metadata(doc)
            res = coll_obj.put(doc, _id=urn)
            return res

    def list(self, wildcard: str, index: str, configuration: str = None):
        """
        List the IDs of the products matching the provided wildcard expression in the given index.

        :param wildcard: wildcard expression
        :param index: the index of the collection in Elasticsearch
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        coll_object = ElasticsearchObject(self._configuration, index)
        res = coll_object.listIDs(wildcard)
        return res

    def ingestFile(self, path=None, schema=None, prod_path=None, prod_filter=None, meta_path=None,
                   meta_key=None, template=None, es_source=False, statistics=None, puregeojson=None, configuration=None):
        """With a folder as main argument, this directive requests the storage of the translate metadatas of products
        in the provided folder (as geojson files).  Flags should be used to specify different information on the source folder.

        :param path: the target folder path where to write results
        :param schema: the path of the geojson schema file against which to validate
        :param prod_path: an iglob expression defining the source folder of the products
        :param prod_filter: an iglob expression to filter files amongst products
        :param meta_path: an iglob expression defining the source folder of the metadatas
        :param meta_key: a regular expression that defines the key to use to match metafile and prodcuts
        :param template: the path to the template file to use for geojson translation
        :param statistics: a boolean indicating if statistics over ingestion must be printed or not
        :param puregeojson: indicates if conversion to pure geojson must be realized or not.
        :param configuration: the path to a file that contains the configuration
        :type path: a string
        :type schema: a string
        :type prod_path: a string
        :type prod_filter: a string
        :type meta_path: a string
        :type meta_key: a string
        :type template: a string
        :type statistics: a boolean
        :type puregeojson: a boolean
        :type configuration: a string
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        folder = self._configuration.file_path if path is None else path
        validator = self._configuration.products_schema if schema is None else schema
        validator = validator if (validator is not None) and (len(validator) > 0) else None
        stat = self._configuration.products_statistics if statistics is None else str(statistics).lower() in ["true", "1"]
        pure = False if puregeojson is None else str(puregeojson).lower() in ["true", "1"]

        if (folder is not None) or (validator is not None):
            from oscars_mgmnt_clms.store.filesystem import GeojsonWriter
            self._result_writer.append(GeojsonWriter(folder, validator, pure))
        if stat:
            from oscars_mgmnt_clms.store.statistics import StatisticsWriter
            self._result_writer.append(StatisticsWriter())
        self._process(prod_path, prod_filter, meta_path, meta_key, template, es_source)

    def ingestKafka(self, topic=None, schema=None, prod_path=None, prod_filter=None, meta_path=None,
                    meta_key=None, template=None, es_source=False, statistics=None, configuration=None):
        """With a topic as argument, this directive requests the sending of the translate metadatas of products
        to a Kafka topic.  Flags should be used to specify different information on the source folder.
        A specific flag allows to define the elasticsearch index to use in the case of a alsatci search consummer of the kafka queue.

        :param topic: Kafka queue where to write results
        :param schema: the path of the geojson schema file against which to validate
        :param prod_path: an iglob expression defining the source folder of the products
        :param prod_filter: an iglob expression to filter files amongst products
        :param meta_path: an iglob expression defining the source folder of the metadatas
        :param meta_key: a regular expression that defines the key to use to match metafile and prodcuts
        :param template: the path to the template file to use for geojson translation
        :param statistics: a boolean indicating if statistics over ingestion must be printed or not
        :param configuration: the path to a file that contains the configuration
        :type topic: a string
        :type schema: a string
        :type prod_path: a string
        :type prod_filter: a string
        :type meta_path: a string
        :type meta_key: a string
        :type template: a string
        :type statistics: a boolean
        :type configuration: a string
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        kafka_topic = self._configuration.kafka_topic if topic is None else topic
        if not kafka_topic:
            raise FireError(errorMessageMandatoryConfig.format("Kafka topic"))
        validator = self._configuration.products_schema if schema is None else schema
        validator = validator if (validator is not None) and (len(validator) > 0) else None
        stat = self._configuration.products_statistics if statistics is None else str(statistics).lower() in ["true", "1"]

        if validator is not None:
            from oscars_mgmnt_clms.store.filesystem import GeojsonWriter
            self._result_writer.append(GeojsonWriter(None, validator))
        if stat:
            from oscars_mgmnt_clms.store.statistics import StatisticsWriter
            self._result_writer.append(StatisticsWriter())
        if kafka_topic is not None:
            from oscars_mgmnt_clms.store.kafka_store import KafkaWriter
            self._result_writer.append(KafkaWriter(self._configuration, kafka_topic))
        self._process(prod_path, prod_filter, meta_path, meta_key, template, es_source)

    def ingestES(self, index=None, bulk_count=None, schema=None, prod_path=None, prod_filter=None, meta_path=None,
                 meta_key=None, template=None, es_source=False, force_index_documents=None, statistics=None, configuration=None):
        """With an elasticsearch index reference as argument followed by the number of entries to ingest
        in each batch, this directive requests the storage of the metadatas of products in the given
        elasticsearch index.  Flags should be used to specify different information on the source folder.

        :param index: the index of the collection
        :param bulk_count: the number of entries to insert per operation
        :param schema: the path of the geojson schema file against which to validate
        :param prod_path: an iglob expression defining the source folder of the products
        :param prod_filter: an iglob expression to filter files amongst products
        :param meta_path: an iglob expression defining the source folder of the metadatas
        :param meta_key: a regular expression that defines the key to use to match metafile and prodcuts
        :param template: the path to the template file to use for geojson translation
        :param es_source: a boolean indicating whether to use Elasticsearch as a metadata source
        :param force_index_documents: a boolean indicating whether to use the 'index' operation for inserting documents in Elasticsearch, rather than the default 'upsert'
        :param statistics: a boolean indicating if statistics over ingestion must be printed or not
        :param configuration: the path to a file that contains the configuration
        :type index: str
        :type bulk_count: int
        :type schema: str
        :type prod_path: str
        :type prod_filter: str
        :type meta_path: str
        :type meta_key: str
        :type template: str
        :type es_source: bool
        :type force_index_documents: bool
        :type statistics: bool
        :type configuration: str
        """
        self._configuration = Config(configuration, (configuration is None) or (self._init_conf))
        es_index = self._configuration.es_index if index is None else index
        if not es_index:
            raise FireError(errorMessageMandatoryConfig.format("ElasticSearch index"))
        es_count = self._configuration.es_bulk_count if bulk_count is None else int(bulk_count)
        force_index_documents = self._configuration.es_force_index_documents if force_index_documents is None else str(force_index_documents).lower() in ["true", "1"]
        if es_count == 1:
            from oscars_mgmnt_clms.store.elasticsearch import EsWriter
            self._result_writer.append(EsWriter(self._configuration, es_index, force_index_documents))
        elif es_count > 1:
            from oscars_mgmnt_clms.store.elasticsearch import EsBulkWriter
            self._result_writer.append(EsBulkWriter(self._configuration, es_index, es_count, force_index_documents))
        validator = self._configuration.products_schema if schema is None else schema
        validator = validator if (validator is not None) and (len(validator) > 0) else None
        stat = self._configuration.products_statistics if statistics is None else str(statistics).lower() in ["true", "1"]
        if validator is not None:
            from oscars_mgmnt_clms.store.filesystem import GeojsonWriter
            self._result_writer.append(GeojsonWriter(None, validator))
        if stat:
            from oscars_mgmnt_clms.store.statistics import StatisticsWriter
            self._result_writer.append(StatisticsWriter())
        self._process(prod_path, prod_filter, meta_path, meta_key, template, es_source)

    def _process(self, prod_path=None, prod_filter=None, meta_path=None, meta_key=None, template=None, es_source=False):
        """This comands requests the process of the ingestion of the different metadafile of products.

        This command must only be used once the other directives have been set.  The following arguments are expected
        ( if they are not already provided in the configuration file ):
        1) prod_path : the filesystem folder where the products are stored.  A * can be used as folder replacer
        to indicates that the system must loop on the parent folder's content.
        2) prod_filter : this optional parameter can be used to filter the files present in the product folder
        ( to avoid the reference of not required text files, ... ).  The argument is present under the form
        of a regular expression.
        3) meta_path : this optional argument indicates that the folder to lookup to find metadatafile.  This
        argument must be used when the product and the metadatiles are not in the same repository.
        4) meta_key : this optional argument can be used when the metadir argument is provided.  It consists
        on a grouped regular expression, that provides the metareference key to use to mach products and their
        associated metadatafiles.  That metareference key is build by applying the regular expression to the
        file names and extring the group definition.
        5) template : this optional argument can be used to request the use of a specific geojson template to
        translate the original xml metadatafile.


        :param prod_path: an iglob expression defining the source folder of the products
        :type prod_path: a string
        :param prod_filter: an iglob expression to filter files amongst products
        :type prod_filter: a string
        :param meta_path: an iglob expression defining the source folder of the metadatas
        :type meta_path: a string
        :param meta_key: a regular expression that defines the key to use to match metafile and prodcuts
        :type meta_key: a string
        :param template: the path to the template file to use for geojson translation
        :type template: a string
        """

        ing_prod_path = self._configuration.products_prod_path if prod_path is None else prod_path
        ing_prod_path = ing_prod_path if (ing_prod_path is not None) and (len(ing_prod_path) > 0) else None
        ing_prod_filter = self._configuration.products_prod_filter if prod_filter is None else prod_filter
        ing_prod_filter = ing_prod_filter if (ing_prod_filter is not None) and (len(ing_prod_filter) > 0) else None
        ing_meta_path = self._configuration.products_meta_path if meta_path is None else meta_path
        ing_meta_path = ing_meta_path if (ing_meta_path is not None) and (len(ing_meta_path) > 0) else None
        ing_meta_key = self._configuration.products_meta_key if meta_key is None else meta_key
        ing_meta_key = ing_meta_key if (ing_meta_key is not None) and (len(ing_meta_key) > 0) else None
        ing_meta_parser = self._configuration.products_meta_parser
        ing_template = self._configuration.products_template if template is None else template
        ing_template = ing_template if (ing_template is not None) and (len(ing_template) > 0) else None
        ing_es_source_host = self._configuration.es_source_host if es_source else None
        ing_es_source_index = self._configuration.es_source_index if es_source else None

        if not ing_prod_path:
            raise FireError(errorMessageMandatoryConfig.format("product path"))
        if len(self._result_writer) == 0:
            print("Nothing to do")
            return

        ingester = MetadataIngester(
            ing_prod_path,
            ing_prod_filter,
            metaparser=ing_meta_parser,
            es_source_host=ing_es_source_host,
            es_source_index=ing_es_source_index)

        if ing_template is not None:
            ingester.with_template(ing_template)
        if ing_meta_path is not None:
            ingester.metaparser.with_meta_directory(ing_meta_path, ing_meta_key)
        ingester.storage = S3Input(self._configuration) if ing_prod_path.startswith("s3://") else FileSystemInput()
        ingester.res_writers = self._result_writer
        ingester.ingest_metadata()
        ingester.flush()


class AllCommands(object):
    """This tool provides facilities to ingest metadatas and manage EO collection.

    This tool provides the required functionalities to
    1) ingest the metadatas of products in specific storage zones (products sub-command),
    2) manage collection definitions of EO products (collections sub-command), and
    3) manage the elasticsearch storage location  (index sub-command).
    4) initialise the oscars system (initialization sub-command)
    5) print the tool version (version sub-command)

     """

    def __str__(self):
        """ blank string representation of the class to avoid
        unfriendly object display
        """
        return ""

    def __init__(self):
        self.products = Products()
        self.collections = Collection()
        self.index = Manage()

    def initialization(self, index=None, configuration=None):
        """ create a collection definition storage in elasticsearch, based
        on the configuration file and on the provided index

        :param index: the index in the elasticsearch where the collection definition must be stored.
        :param configuration: the path to a file that contains the configuration
        :type index: a string
        :type configuration: a string
        """
        self.index._initoscars(index, configuration)

    def version(self):
        """This comands print the version number and the date of building of the current tool.
        """
        print("OSCARS Management Command Line Tool version {}".format(oscars_mgmnt_clms.__version__))


if __name__ == "__main__":
    import logging

    logging.basicConfig()
    logging.root.setLevel(logging.INFO)
    import fire

    fire.Fire(AllCommands)
