"""
Created on 02 July 2019

@author: Tonneau Stephane
@author: Dirk Daems
"""
from typing import Optional
import oscars_mgmnt_clms.utils.storage
from oscars_mgmnt_clms.file_object import ObjectInfo


class MetadataIngester(object):
    """Class that is responsible for the ingestion of EO product metadata"""

    def __init__(self, prod_path, product_filter, metaparser=None, es_source_host=None, es_source_index=None):
        """
        Constructor of the class.
        """
        self.prod_path = prod_path
        self.prod_filter = product_filter

        self.metaparser = metaparser

        if self.metaparser is None:
            #if any(path_substr in self.prod_path for path_substr in ["ndvi_300m_v2_10daily"]):
            from oscars_mgmnt_clms.parser.filename import CLMSGlobalParser_netCDF, CLMSGlobalParser_GeoJSON
            if 'water_level' in self.prod_path.lower(): 
                self.metaparser = CLMSGlobalParser_GeoJSON()
            else:
                self.metaparser = CLMSGlobalParser_netCDF()

        self.res_writers = []
        self.storage: Optional[oscars_mgmnt_clms.utils.storage.StorageInput] = None

    def with_template(self, template):
        """Method that fixes the jinja2 template to use in the process
        of translation of xml files to geojson files.

        :param template:the path to a jinja2 template.
        :type template: a string
        :returns: the object itself
        :rtype: a MetadataIngester
        """
        if self.metaparser is None:
            return None
        if template is not None:
            self.metaparser.with_template(template)
        return self

    def with_writer(self, writer):
        """Method that appends a new storage writing component to the list
        of referenced storage location.

        :param writer: a storage location writer such as FileWriter, KafkaWriter, ...
        :type writer: an OutputWriter
        :returns: the object itself
        :rtype: a MetadataIngester
        """
        self.res_writers.append(writer)
        return self

    def ingest_metadata(self):
        """Method that launches the process of
        - product path analysis
        - xml metadata translation to geojson equivalence
        - storing of the geojson files in the configured storage locations.
        """
        # Checking the parser
        if self.metaparser is None:
            raise Exception('No template available for parsing')

        # Reading the file system
        path_cnt = 0
        product_cnt = 0
        for filename in self.storage.glob(self.prod_path):
            path_cnt += 1
            product_file_infos = ObjectInfo.from_regexp(self.storage, filename, self.prod_filter)
            for file_infos in product_file_infos.items():
                filled_templates = self.metaparser.convert(file_infos[1])
                for uid in filled_templates:
                    product_cnt += 1
                    for writer in self.res_writers:
                        writer.output(uid, filled_templates[uid])
        if path_cnt < 1 or product_cnt < 1:
            from oscars_mgmnt_clms.utils.common import last_iglob
            from fire.core import FireExit
            last_iglob("No entries found.  Best matching product entry is", self.prod_path)
            # exit with status code 3 (exit code 2 is default error code for Fire)
            raise FireExit(3, None)

    def flush(self):
        """Method that flushes the storage queue of all referenced storage systems.
        """
        for writer in self.res_writers:
            writer.flush()
