"""
Created on 30 June 2019

@author: Tonneau Stephane
"""

import os
import time
import uuid
import glob
import re
import logging
import xml.etree.ElementTree
from jinja2 import Environment, BaseLoader
from oscars_mgmnt_clms.utils.common import ResourceReader
from oscars_mgmnt_clms.file_object import ObjectInfo

METAPARSER_LOGGER = logging.getLogger('MetaParser')


class SpecificInfo(object):
    """class that store an association of a file description
    object and a dictionary"""

    def __init__(self, uid, product_file_info, values):
        """ Initialisator of the class
        """
        self.uid = uid
        self.product_file_info = product_file_info
        self.values = values


class BaseParser(object):
    """ Parser for metadata """

    def __init__(self, template=None):
        self.dict_info = {}
        self.init_dict()
        self.jinja_template = None
        self.template = template

    def init_dict(self):
        self.dict_info = {}

    def with_template(self, template):
        """ method that fixes the jinja2 template to use

        :param template: the template to fill in
        :type template: str
        :returns: the object itself
        :rtype: BaseParser
        """
        template_file = template
        if not os.path.isabs(template):
            root = os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir))
            template_file = os.path.join(os.path.join(root, 'resources/templates'), template)
        template_string = ResourceReader.get_reader(template_file).get_string()
        jinja_env = Environment(loader=BaseLoader)
        self.jinja_template = jinja_env.from_string(template_string)
        return self

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):
        """ method that should be overwritten to extract some specific information
        or to perform some specific computation for a single metadafile creation.

        :param product_file_info: the description of the files associated to a product
        :type product_file_info:
        :param values: the key value association to update
        :type values: dict
        :returns: the id of the product
        :rtype: str
        """
        return str(uuid.uuid1()) if ("id" not in values) or (values["id"] is None) else values["id"]

    def splitted_treatment(self, product_file_info, values):
        """ method that should be overwritten to extract some specific information
        or to perform some specific computation for several metadafiles creation.

        :param product_file_info: the description of the files associated to a product
        :type product_file_info: a file object info
        :param values: the key value aossication to update
        :type values: dict
        :returns: a list of SpecificInfo
        :rtype: list
        """
        uid = self.specific_treatment(product_file_info, values)
        return [SpecificInfo(uid, product_file_info, values)]

    def convert(self, product_file_info):
        """ Convert the metadata to the desired product metadata used for ingestion
        :param product_file_info: the description of the files associated to a product.
        :type product_file_info: ObjectInfo
        """
        raise NotImplementedError("Implement this method in a subclass")


class XMLMetaParser(BaseParser):
    """class that parse an object to retrieve a metafile"""

    base_key = r".*[^0-9a-zA-Z]2[0-4](\d\d\d\d\d\d)[^0-9a-zA-Z][^V]*[V](\d\d\d).*"
    ext_key = r".*[^0-9a-zA-Z]2[0-4](\d\d\d\d\d\d)[^0-9a-zA-Z](\d\d\d\d\d\d)[^V]*[V](\d\d\d).*"
    full_key = r".*[^0-9a-zA-Z]2[0-4](\d\d\d\d\d\d)[^0-9a-zA-Z](\d\d\d\d\d\d)[^0-9a-zA-Z](\d)[^V]*[V](\d\d\d).*"
    L1C_key = r".*_2\d(\d\d\d\d\d\d)T\d\d\d\d\d\d_([0-9a-zA-Z]*)_([0-9a-zA-Z]*)_([0-9a-zA-Z]*).*"

    def __init__(self, template=None):
        """ Initialisator of the class
        """
        super().__init__(template)
        self.metafiles = {}
        self.mf_regexp = None
        self.namespace = {}

    def with_meta_directory(self, meta_dir=None,
                            meta_key=None):
        """ method that fixes a specific metadata directory to search when
        a metadatafile is not part of the product directory.

        :param meta_dir: the directory that contains the metadafiles
        :type meta_dir: a string
        :param meta_key: a regular expression (with group definition) to capture the reference key
        :type meta_key: a string
        :returns: the object itself
        :rtype: a MetaParser
        """
        if meta_dir is not None:
            self.mf_regexp = XMLMetaParser.base_key if meta_key is None else meta_key
            if (self.mf_regexp == "extended"):
                self.mf_regexp = XMLMetaParser.ext_key
            elif (self.mf_regexp == "full"):
                self.mf_regexp = XMLMetaParser.full_key
            elif (self.mf_regexp == "base"):
                self.mf_regexp = XMLMetaParser.base_key
            elif (self.mf_regexp == "L1C"):
                self.mf_regexp = XMLMetaParser.L1C_key

            for filename in glob.iglob(meta_dir):
                if os.path.isfile(filename):
                    match = re.search(self.mf_regexp, filename)
                    if ((match) and (len(match.groups()) > 0)):
                        nbr_matches = len(match.groups()) + 1
                        uid = match.group(1)
                        for pos in range(2, nbr_matches):
                            uid = uid + match.group(pos)
                        if uid in self.metafiles:
                            METAPARSER_LOGGER.error("Duplicate %s key present in metafilter", uid)
                            raise Exception("Duplicate " + uid + " key present in metafilter")
                        self.metafiles[uid] = filename
        return self

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict, xmltree=None):
        """ method that should be overwritten to extract some specific information
        or to perform some specific computation for a single metadafile creation.

        :param product_file_info: the description of the files associated to a product
        :type product_file_info: a file object info
        :param values: the key value association to update
        :type values: a dictionary
        :param xmltree: the xml tree representation of the metaile
        :type xmltree: an xml tree
        :returns: the values dictionary filled with specific values
        :rtype: a dictionary
        """
        return str(uuid.uuid1()) if ("id" not in values) or (values["id"] is None) else values["id"]

    def splitted_treatment(self, product_file_info, values, xmltree=None):
        """ method that should be overwritten to extract some specific information
        or to perform some specific computation for several metadafiles creation.

        :param product_file_info: the description of the files associated to a product
        :type product_file_info: a file object info
        :param values: the key value aossication to update
        :type values: a dictionary
        :param xmltree: the xml tree representation of the metaile
        :type xmltree: an xml tree
        :returns: a list of SpecificInfo
        :rtype: a list of SpecificInfo
        """
        uid = self.specific_treatment(product_file_info, values, xmltree)
        return [SpecificInfo(uid, product_file_info, values)]

    def convert(self, product_file_info, metafile=None):
        """ uses the metadata file present in product_file_info
        and the files description present in the same structure into
        a new metadatafile whose template is provided at class initialisation time.

        :param product_file_info: the description of the files associated to a product
        :type product_file_info: a file object info
        :returns: the template filled in with the specific values
        :rtype: a string
        """

        metadata_file = None
        if metafile is not None:
            metadata_file = metafile
        elif product_file_info is None:
            raise Exception("No metadata file available")
        elif product_file_info.get_metafileinfo() is not None:
            metadata_file = product_file_info.get_metafileinfo().name
        elif self.mf_regexp is not None:
            tmp_name = product_file_info.get_product_path()
            match = re.search(self.mf_regexp, tmp_name)
            if ((match) and (len(match.groups()) > 0)):
                uid = match.group(1)
                nbr_matches = len(match.groups()) + 1
                for pos in range(2, nbr_matches):
                    uid = uid + match.group(pos)
                if uid in self.metafiles:
                    metadata_file = self.metafiles[uid]
                    # add metadata file to product_file_info
                    product_file_info.add(metadata_file)
                else:
                    METAPARSER_LOGGER.error("Missing key %s in provided meta directory for analysis of %s",
                                            uid, product_file_info.get_product_path())
                    return {}
        if metadata_file is None:
            if product_file_info.size() == 0:
                METAPARSER_LOGGER.info("No files found for directory %s",
                                       product_file_info.get_product_path())
                return {}
            raise Exception("No metadata file found to analyse {}"
                            .format(product_file_info.get_product_path()))
        if self.jinja_template is None:
            self.with_template(self.template)
        xml_tree = None
        try:
            xml_tree = xml.etree.ElementTree.parse(metadata_file).getroot()
            self.namespace = self.parse_xml_namespace(metadata_file)
        except Exception as err:
            METAPARSER_LOGGER.error("Metafile parsing exception for %s",
                                    str(err))
            raise

        the_values = {}
        for key, keyval in self.dict_info.items():
            try:
                result = xml_tree.findall(keyval, self.namespace)
                if (result is not None) and (len(result) > 0) and (result[0].text is not None):
                    the_values[key] = str(result[0].text)
            except Exception as err:
                METAPARSER_LOGGER.error("Key analysis of %s (%s) : %s",
                                        key, keyval,
                                        str(err))
                raise

        # specific metadatas
        if product_file_info is None:
            raise Exception("No product file description provided")
        product_path = product_file_info.get_product_path()

        the_values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        the_values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())

        specific_infos = self.splitted_treatment(product_file_info, the_values, xml_tree)

        # Generation of the output
        results = {}
        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path(),
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related
            )
            results[specific.uid] = result

        return results

    def parse_xml_namespace(self, filename):
        namespace = {}
        for _, elem in xml.etree.ElementTree.iterparse(filename, ["start-ns"]):
            namespace[elem[0]] = elem[1]
        return namespace
