from oscars_mgmnt_clms.file_object import ObjectInfo
from oscars_mgmnt_clms.parser.base import BaseParser
from datetime import datetime as dt

class CLMSGlobalParser_netCDF(BaseParser):
    """ Parser used to build GeoJSON metadata from CLMS Global Land products in netCDF format"""

    def __init__(self):
        super().__init__(template='CLMS_GLOBAL.template')

    def __decodePlatforms__(self,platformstr):
        import os, re
        # decodes the netCDF :platform attribute into a list of one or more
        # platforms, with consistent formatting

        # examples:
        # Proba-V => [PROBA-V]
        # SPOT 4 => [SPOT-4]
        # METOP A+B => [METOP-A, METOP-B]
        # Sentinel-1 A+B => [SENTINEL-1A, SENTINEL-1B]
        # METOP A+B, Sentinel-1 A+B => [METOP-A, METOP-B, SENTINEL-1A, SENTINEL-1B]
        # SENTINEL-3A AND SENTINEL-3B => [SENTINEL-3A, SENTINEL-3B]
        # SENTINEL3A AND SENTINEL3B => [SENTINEL-3A, SENTINEL-3B]
        # SENTINEL 3 (A AND B) => [SENTINEL-3A, SENTINEL-3B]
        # TERRA (EOS AM-1) => [TERRA]
        # N/A => [N/A]

        platformstr=platformstr.upper().replace("_","-").replace('SUOMI-NPP','SUOMI NPP').strip()

        # support for platforms with LIE 250m with old name for TERRA/AQUA
        # TERRA (EOS AM-1) => [TERRA]
        for modis in ["AQUA","TERRA"]:
            if modis in platformstr and ' (' in platformstr:
                platformstr=platformstr.split(' (')[0]

        split_comma=platformstr.split(",")
        platforms=[x.strip() for x in split_comma]

        # Support for "AND" between platform names or platform serials
        # e.g. "SENTINEL-3A AND SENTINEL-3B" or "SENTINEL 3 (A AND B)"
        bracket_serials_regex="\(([A-Z]) AND ([A-Z])\)" 
        if ' AND ' in platformstr:
            repl_platforms=[]
            for platf in platforms:
                regex_match_part=re.search(bracket_serials_regex,platf) 
                if regex_match_part:
                    (platformbase,platformserial)=platf.split(" (")
                    platformbase=platformbase.replace(" ","-")
                    sep="-"
                    if "-" in platformbase:
                        sep="" 
                    repl_platforms.extend([platformbase+sep+serial for serial in regex_match_part.groups()])
                else:
                    repl_platforms.extend(platf.split(" AND "))
            platforms=repl_platforms

        # Support for SSM/SWI multi-mission products with eg "METOP B+C"
        if "+" in platformstr:
            repl_platforms=[]
            for platf in platforms:
                if "+" in platf:
                    (platformbase,platformserial)=platf.split(" ")
                    split_serial=platformserial.split("+")
                    sep="-"
                    if "-" in platformbase:
                       sep="" 
                    repl_platforms.extend([platformbase+sep+serial for serial in split_serial])
                else:
                    repl_platforms.extend(platf)
            platforms=repl_platforms

        # We covered the AND and + syntaxes, all remaining spaces are between platforms
        # with exception of "SUOMI NPP"
        repl_platforms=[] 
        for platf in platforms:
            if ' ' in platf and not 'SUOMI NPP' == platf:
                repl_platforms.extend(platf.split(' '))
            else:
                repl_platforms.append(platf)
            platforms=repl_platforms

        # Support for LST, eg MSG4 -> MSG-4
        geos=["GOES","MSG","HIMAWARI","MTSAT"]
        lst_regex="(MSG|MTSAT|HIMAWARI|GOES)([0-9]+|E|R)"
        if any([x in platformstr for x in geos]):
            repl_platforms=[] 
            for platf in platforms:
                regex_match=re.match(lst_regex,platf)
                if regex_match:
                    repl_platforms.append('-'.join(regex_match.groups()))
                else:
                    repl_platforms.append(platf)
            platforms=repl_platforms

        # Hyphenate SENTINELs, eg SENTINEL3A => SENTINEL-3A
        sentinel_regex="(SENTINEL)([0-6][A-Z]+)"
        if 'SENTINEL' in platformstr:
            repl_platforms=[] 
            for platf in platforms:
                if 'SENTINEL' in platf:
                    regex_match=re.match(sentinel_regex,platf)
                    if regex_match:
                        repl_platforms.append('-'.join(regex_match.groups()))
                    else:
                        repl_platforms.append(platf)
                else:
                    repl_platforms.append(platf)
            platforms=repl_platforms

        return platforms

    def convert(self, product_file_info):

        import os,re
        import time
        import numpy as np
        from netCDF4 import Dataset

        # Initialize values dict
        values = {}

        # examples of supported filenames:
        #    c_gls_NDVI300_202204110000_GLOBE_OLCI_V2.0.1.nc
        #    c_gls_BA300-NRT_201901010000_GLOBE_S3_V3.1.1.nc
        fn = os.path.abspath(product_file_info.get_product_path())

        ds = Dataset(fn)

        # filename without extension
        file_name = os.path.basename(fn).rpartition('.')[0]
        file_name_parts = file_name.split('_')

        nominaldatetime = self.parse_fname_date(file_name_parts[3])
        values['nominaldatetime'] = nominaldatetime.isoformat()+'Z'

        (product_type,separator,product_group) = file_name_parts[2].partition('-')
        product_group=product_group.replace("STATIC-","")
        if product_group:
            values['productgroupid']=product_group

        processingCenter='VITO'
        if 'institution' in ds.ncattrs():
            processingCenter = ds.institution.split(',')[0].upper()
        elif 'processing_centre' in ds.ncattrs():
            processingCenter = ds.processing_centre.split(',')[0].upper()
        values['processingCenter']=processingCenter.replace(' NV','').replace(' GMBH','').replace(' ','_')

        version = file_name_parts[-1]
        # major_version = int(version[1])

        tile_id = file_name_parts[-3]

        # collection identifier is constructed from the collection-folder
        path_elem=os.path.dirname(fn).split(os.path.sep)

        # assuming files are stored ...
        nominal_year_str=str(nominaldatetime.year)
        #  ...  either in 3rd level below collection, ie collection/yyyy/yyyymmdd/file
        if path_elem[-2] == nominal_year_str and path_elem[-1].startswith(nominal_year_str):
            collection_dir=path_elem[-3]
        else:
            #  ...  or in lowest folder level (no organization per date, e.g. static masks)
            collection_dir=path_elem[-1]

        parent_identifier = '_'.join(['clms_global',collection_dir,'netcdf'])
        values['parentidentifier'] = parent_identifier

        values['id'] = file_name
        values['title'] = ds.title

        # TODO extend for SWI DGG product
        values['conformsto'] = 'https://www.opengis.net/def/crs/EPSG/0/4326'

        # Coordinate rounding to 10 decimals
        # Allow one more digit for LST to avoid ES map parser error 
        rounding=10
        if 'land_surface_temperature' in fn:
            rounding=11

        # netCDF stores center of pixel coordinates, OSCARS stores pixel top left coordinates
        # so we need to adjust with half a pixel
        westcenter=np.min(ds.variables['lon'])
        southcenter=np.min(ds.variables['lat'])
        eastcenter=np.max(ds.variables['lon'])
        northcenter=np.max(ds.variables['lat'])

        if 'TS' == product_group:
            # SWI Time series: DGG grid is irregular in lat/lon degrees, no half pixel adjustment
            values['westbbox'] = np.round(westcenter,rounding)
            values['southbbox'] = np.round(southcenter,rounding)
            values['eastbbox'] = np.round(eastcenter,rounding)
            values['northbbox'] = np.round(northcenter,rounding)
        else:
            rasterxsize=len(ds.variables['lon'])
            rasterysize=len(ds.variables['lat'])

            step=(eastcenter-westcenter)/(rasterxsize-1)
            # improve final decimals for SWI and PROBA-V/Sentinel-3 vegetation products
            for realstep in [.1, 1./112, 1./336]:
                precision=10
                if (np.round(realstep,precision)==np.round(step,precision)): 
                    step=realstep
                    eastcenter=westcenter+((rasterxsize-1)*step)
                    southcenter=northcenter-((rasterysize-1)*step)
            halfpixel=step/2.

            # shift half pixel towards from center to top-left of pixel
            values['westbbox'] = np.round(westcenter-halfpixel,rounding)
            values['southbbox'] = np.round(southcenter-halfpixel,rounding)
            values['eastbbox'] = np.round(eastcenter+halfpixel,rounding)
            values['northbbox'] = np.round(northcenter+halfpixel,rounding)

        # LWQ refers to GLOBAL, rename to GLOBE for consistency with other products
        values['tileid'] = tile_id.replace('GLOBAL','GLOBE')

        # collection_dir can have 12.5km, 5km, 1km, 300m, 100m or no resolution info
        resolution_regex=re.compile('([\\d.]+)(m|km)')
        regex_result = resolution_regex.match(collection_dir.split('_')[1])

        if regex_result and not product_group in ['DGG','TS']:
            regex_result = resolution_regex.match(collection_dir.split('_')[1])
            factor = 1000 if regex_result.group(2)[0]=='k' else 1
            values['resolution'] = int(float(regex_result.group(1))*factor)
        else:
            values['resolution'] = None

        values['producttype'] = product_type
        values['productVersion'] = version
        #values['platformShortName'] = ', '.join([x.strip().upper().replace("_","-") for x in ds.platform.split(',')])

        if 'time_coverage_start' in ds.ncattrs():
            values['beginposition'] = ds.time_coverage_start
            values['endposition'] = ds.time_coverage_end
        else:
            values['beginposition'] = nominaldatetime.isoformat()+'Z'
            values['endposition'] = nominaldatetime.isoformat()+'Z'

        values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['updatetime'] = time.strftime(
            '%Y-%m-%dT%H:%M:%SZ',
            product_file_info.storage.get_time_modified(fn)
        )

        if self.jinja_template is None:
            self.with_template(self.template)

        specific_infos = self.splitted_treatment(product_file_info, values)

        # Generation of the output
        results = {}
        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path().rpartition('/')[0],
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related,
                platforms=self.__decodePlatforms__(ds.platform)
            )
            results[specific.uid] = result
        return results

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):
        self.fix_products(product_file_info)
        return values['id']

    @staticmethod
    def fix_products(fileobject):

        import os, re

        for info in list(fileobject.related):
            if info.name.endswith('.nc'):
                title=os.path.basename(info.name).split('_')[2].split('-')[0]
                re_match = re.search("\d", title)
                if re_match:
                    #if needed, use all characters up to first digit (eg NDVI300 -> NDVI)
                    title=title[0:re_match.start()]

                info.title = title
                info.type  = 'application/netcdf'
                fileobject.related.remove(info)
                fileobject.datas.append(info)

    # Nominal dates in file names can take one of three forms:
    #    YYYYMMDDhhmm (12 digits): the most common case
    #    YEAR1----YEAR2 (4 digits, 4 hyphens, 4 digits) -> nominal date = Jan 1 of YEAR1:
    #      e.g. SWI static
    #    YEAR1-YEAR2-MMDD (4 digits, hyphen, 4 digits, hyphen, 4 digits) -> nominal date = YEAR1/MM/DD: 
    #      e.g. NDVI statistics
    def parse_fname_date(self,fname_date):
        import re
        if re.search("[0-9]{12}", fname_date):
            return dt.strptime(fname_date,'%Y%m%d%H%M')
        elif re.search("[0-9]{4}----[0-9]{4}", fname_date):
            return dt.strptime(fname_date[0:4],'%Y')
        else:
            return dt.strptime(fname_date[0:4]+fname_date[-4:],'%Y%m%d')

class CLMSGlobalParser_GeoJSON(BaseParser):
    """ Parser used to build GeoJSON metadata from CLMS Global Land products in GeoJSON format (mainly Lake and River Water Level) """

    def __init__(self):
        super().__init__(template='CLMS_GLOBAL_point.template')

    def convert(self, product_file_info):
        import os, re
        import string
        import time
        import numpy as np
        import geojson

        # Initialize values dict
        values = {}

        # examples of supported filenames:
        #    c_gls_WL_202310182114_1300000001574_ALTI_V2.2.0.json (lake)
        #    c_gls_WL_202310182333_0000000112081_ALTI_V2.2.0.json (river)
        fn = os.path.abspath(product_file_info.get_product_path())

        with open(fn) as jsonfile:
            json=geojson.load(jsonfile)

        # filename without extension
        file_name = os.path.basename(fn).rpartition('.')[0]
        file_name_parts = file_name.split('_')

        nominaldatetime = self.parse_fname_date(file_name_parts[3])
        values['nominaldatetime'] = nominaldatetime.isoformat()+'Z'

        values['productgroupid']=string.capwords(json.properties['basin'])

        values['processingCenter'] = 'CLS'
        if 'institution' in json.properties:
            if not json.properties['institution'] == 'LEGOS, CLS, CNES':
                values['processingCenter'] = json.properties['institution'].split(',')[0].upper()

        version = file_name_parts[-1]
        # major_version = int(version[1])

        if 'lake' in json.properties:
            tile_id = string.capwords(json.properties['lake'])
        else:
            tile_id = string.capwords(json.properties['river'])

        # collection identifier is constructed from the collection-folder
        path_elem=os.path.dirname(fn).split(os.path.sep)

        # assuming files are stored ...
        nominal_year_str=str(nominaldatetime.year)
        #  ...  either in 3rd level below collection, ie collection/yyyy/yyyymmdd/file
        if path_elem[-2] == nominal_year_str and path_elem[-1].startswith(nominal_year_str):
            collection_dir=path_elem[-3]
        else:
            #  ...  or in lowest folder level (no organization per date, e.g. static masks)
            collection_dir=path_elem[-1]

        parent_identifier = '_'.join(['clms_global',collection_dir,'geojson'])

        values['parentidentifier'] = parent_identifier

        # JSON files get updates on every overpass. Filename stores the last observation timestamp.
        # To facilitate the updates of JSON metadata in ES, the timestamp is removed from the id
        del file_name_parts[3]
        values['id'] = '_'.join(file_name_parts)

        # no title property in GeoJSON, so let's build a reasonable title
        if 'lake' in json.properties:
            values['title'] = ' '.join(['Lake water level time series:',json.properties['basin'],json.properties['lake']])
        else:
            values['title'] = ' '.join(['River water level time series:',json.properties['basin'],json.properties['river']])

        values['conformsto'] = 'https://www.opengis.net/def/crs/EPSG/0/4326'

        # Lake and River water level are encoded as observation time series for a specific point location, not a raster
        # shift half pixel towards from center to top-left of pixel
        (lon,lat)=json.geometry.coordinates
        values['point_lon'] = lon
        values['point_lat'] = lat

        if 'lake' in json.properties:
            values['tileid'] = json.properties['lake']
        else:
            values['tileid'] = json.properties['river']

        values['producttype'] = 'WL'

        # Lake and River water level are encoded as observation time series for a specific point location, not a raster
        values['resolution'] = None

        values['productVersion'] = version

        if 'time_coverage_start' in json.properties:
            values['beginposition'] = time.strftime("%Y-%m-%dT%H:%M:%SZ",time.strptime(json.properties['time_coverage_start'],"%Y-%m-%d %H:%M:%S"))
            values['endposition'] = time.strftime("%Y-%m-%dT%H:%M:%SZ",time.strptime(json.properties['time_coverage_end'],"%Y-%m-%d %H:%M:%S"))
        else:
            values['beginposition'] = nominaldatetime.isoformat()+'Z'
            values['endposition'] = nominaldatetime.isoformat()+'Z'

        values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['updatetime'] = time.strftime(
            '%Y-%m-%dT%H:%M:%SZ',
            product_file_info.storage.get_time_modified(fn)
        )

        if self.jinja_template is None:
            self.with_template(self.template)

        specific_infos = self.splitted_treatment(product_file_info, values)

        # Generation of the output
        results = {}
        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path().rpartition('/')[0],
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related,
                platforms=['Altimeters']
            )
            results[specific.uid] = result
        return results

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):
        self.fix_products(product_file_info)
        return values['id']

    @staticmethod
    def fix_products(fileobject):

        import os, re

        for info in list(fileobject.related):
            if info.name.endswith('.json'):
                title=os.path.basename(info.name).split('_')[2].split('-')[0]
                re_match = re.search("\d", title)
                if re_match:
                    #if needed, use all characters up to first digit (eg NDVI300 -> NDVI)
                    title=title[0:re_match.start()]

                info.title = title
                info.type  = 'application/geo+json'
                fileobject.related.remove(info)
                fileobject.datas.append(info)

    # Nominal dates in file names can take one of three forms:
    #    YYYYMMDDhhmm (12 digits): the most common case (used for Water Level GeoJSON)
    #    YEAR1----YEAR2 (4 digits, 4 hyphens, 4 digits) -> nominal date = Jan 1 of YEAR1:
    #      e.g. SWI static
    #    YEAR1-YEAR2-MMDD (4 digits, hyphen, 4 digits, hyphen, 4 digits) -> nominal date = YEAR1/MM/DD: 
    #      e.g. NDVI statistics
    def parse_fname_date(self,fname_date):
        import re
        if re.search("[0-9]{12}", fname_date):
            return dt.strptime(fname_date,'%Y%m%d%H%M')
        elif re.search("[0-9]{4}----[0-9]{4}", fname_date):
            return dt.strptime(fname_date[0:4],'%Y')
        else:
            return dt.strptime(fname_date[0:4]+fname_date[-4:],'%Y%m%d')

if __name__ == "__main__":
    import sys,os
    from oscars_mgmnt_clms.utils.storage import FileSystemInput
    import json

    if len(sys.argv)>1:
        product_path = os.path.abspath(sys.argv[1])
    else:
        product_path = '/data/MTDA/Copernicus/Land/global/netcdf/ndvi/ndvi_300m_v2_10daily/2022/20220411' \
                       '/c_gls_NDVI300_202204110000_GLOBE_OLCI_V2.0.1.nc'
    product_file_info = ObjectInfo.from_regexp(FileSystemInput(), product_path)
    if 'water_level' in product_path.lower():
        parser = CLMSGlobalParser_GeoJSON()
    else:
        parser = CLMSGlobalParser_netCDF()
    results = parser.convert(list(product_file_info.values())[0])
    json_data = json.loads(next(iter(results.values())))
    print(json.dumps(json_data,indent=4))
