"""
Created on 5 augustus 2019

@author: Tonneau Stephane
@author: Dirk Daems
"""

import logging
from oscars_mgmnt_clms.utils.common import OutputWriter
KAFKA_LOGGER = logging.getLogger('KafkaWriter')

try:
    from kafka import KafkaProducer
except ImportError:
    # the unavailability of the package shall be treated later
    # when trying to use a producer
    KAFKA_LOGGER.fatal("Unable to import kafka library")
    raise


def to_topic(source):
    """ return the string as a byte encoded list of characters for python 3.x"""
    return bytes(source, 'utf-8')


class KafkaWriter(OutputWriter):
    """Writer that published metadata documents on a kafka topic (document by document)."""

    def __init__(self, config, topic):
        """ Constructor of the class
        :param config: A configuration object
        :type config: a Config object
        """
        OutputWriter.__init__(self)
        self.config = config
        self.producer = None
        self.kafka_topic = self.config.kafka_topic if topic is None else topic
        try:
            if self.config.kafka_secure is None:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.config.kafka_brokers, acks='all')
            else:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.config.kafka_brokers,
                    security_protocol='SASL_PLAINTEXT',
                    sasl_mechanism='GSSAPI', acks='all')
            if self.producer is None:
                KAFKA_LOGGER.error("Kafka producer NOT created!")
        except (Exception) as error:
            KAFKA_LOGGER.error("Unable to create a Kafka producer for brokers %s, mode %s: %s",
                               str(self.config.kafka_brokers),
                               str(self.config.kafka_secure),
                               str(error))
            self.producer = None
            raise error

    def close(self):
        """Closes the kafka producer."""
        if self.producer is not None:
            try:
                self.producer.flush()
                self.producer.close()
            except (Exception) as error:
                KAFKA_LOGGER.error("Unable to close Kafka producer for brokers %s, mode %s (%s)",
                                   str(self.config.kafka_brokers),
                                   self.config.kafka_secure, str(error))
                raise error
            finally:
                self.producer = None

    def output(self, uid, content):
        """ Writes the content to a kafka topic.

        :param uid: a unique reference
        :type uid: a string
        :param content: a string containing the metadata content to save.
        :type content: a string
        """
        if self.producer is not None:
            try:
                message = content
                future = self.producer.send(
                    self.kafka_topic, to_topic(message))
                future.get(timeout=60)
            except (Exception) as error:
                KAFKA_LOGGER.error("Unable to publish Kafka message for brokers %s, topic %s, mode %s: %s",
                                   str(self.config.kafka_brokers),
                                   str(self.kafka_topic),
                                   str(self.config.kafka_secure),
                                   str(error))
                raise error
