"""
Created on 30 June 2019

@author: Tonneau Stephane
"""

import os
import sys
import logging
import json
import zipfile
import copy
from jsonschema import validate
import glob

CONFIG_LOGGER = logging.getLogger('Config')
OUTPUT_LOGGER = logging.getLogger('OutputWriter')
resource_path = os.path.normpath(os.path.join(os.path.split(__file__)[0], "../resources"))


def last_iglob(message, path):
    new_path = path
    last = len(path)
    while last > 0:
        new_path = path[:last]
        for filename in glob.iglob(new_path):
            OUTPUT_LOGGER.error(message+" "+str(new_path))
            return
        last = new_path.rindex('/')


class FileReader(object):
    """Class that is used to read text files from file system.
    """

    def __init__(self, filename):
        self.filename = filename

    def isfile(self):
        """check if the provided filename is a file or not.

        :returns: True if the file really exists; False otherwise
        :rtype: a boolean
        """
        return os.path.isfile(self.filename)

    def get_string(self):
        """Read the content of the file and returns its string representation.

        :returns: the text content of the file
        :rtype: a string
        """
        data = None
        with open(self.filename, 'r') as file:
            data = file.read()
        return data

    def get_file(self):
        """Read the content of the file and returns it as a file descriptor.

        :returns: the text content of the file as a file descriptor
        :rtype: a file descriptor
        """
        import StringIO
        data = self.get_string()
        return StringIO.StringIO(data)


class ZipReader(FileReader):
    """Class that is used to read text files from zip archive.
    """
    def __init__(self, filename):
        FileReader.__init__(self, filename)
        pos = filename.find(".zip")
        self.zipbase = filename[0:pos+4]
        self.zipfile = filename[pos+5:].replace("\\", "/")

    def isfile(self):
        """check if the provided filename exists or not.

        :returns: True if the file really exists; False otherwise
        :rtype: a boolean
        """
        with zipfile.ZipFile(self.zipbase) as zf:
            return self.zipfile in zf.namelist()

    def get_string(self):
        """Read the content of the file and returns its string representation.

        :returns: the text content of the file
        :rtype: a string
        """
        data = None
        with zipfile.ZipFile(self.zipbase) as zf:
            zfile = zf.open(self.zipfile, "r")
            data = zfile.read().decode('ascii')
        return data


class PkgReader(FileReader):
    """Class that is used to read text files from a python archive.
    """
    def __init__(self, filename):
        FileReader.__init__(self, filename)
        pos = filename.find(".pyz")
        tmp_name = filename[pos+5:].replace("\\", "/")
        pos = tmp_name.find("/")
        self.pkgbase = tmp_name[:pos]
        self.pkgfile = tmp_name[pos+1:]
        import pkg_resources
        self.rs = pkg_resources.ResourceManager()

    def isfile(self):
        """check if the provided filename exists or not.

        :returns: True if the file really exists; False otherwise
        :rtype: a boolean
        """
        return self.rs.resource_exists(self.pkgbase, self.pkgfile)

    def get_string(self):
        """Read the content of the file and returns its string representation.

        :returns: the text content of the file
        :rtype: a string
        """
        return self.rs.resource_string(self.pkgbase, self.pkgfile).decode('ascii')


class ResourceReader(object):
    """Factory class that returns the correct reader according the filename.
    """

    @classmethod
    def get_reader(cls, filename):
        """returns the correct reader according the path of the filename.

        :param filename: the filename
        :type filename: a string
        :returns: the text content of the file
        :rtype: a Reader
        """
        if ".zip" in filename:
            return ZipReader(filename)
        elif ".pyz" in filename:
            return PkgReader(filename)
        return FileReader(filename)


class JsonValidator(object):
    """Class that is used to validate geoJson."""

    def __init__(self, filename):
        self.schema = None
        if filename is not None:
            if os.path.isabs(filename):
                validator_file = filename
            else:
                root = os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                                     os.pardir))
                validator_file = os.path.join(os.path.join(root, 'resources'), filename)
            self.schema = json.loads(ResourceReader.get_reader(validator_file).get_string())

    def check(self, content):
        """validates a json structure according the provided schema.

        :param content: the json structure to validate
        :type content: a json structure
        """
        validate(content, self.schema)


class Jsonify(object):
    """Class that is used to translate esJson to geoJson."""

    def __init__(self, content=None, validator=None):
        self.content = None if content is None else json.loads(content)
        self.validator = validator

    def __str__(self):
        return json.dumps(self.content, indent=4)

    def to_geoJson(self):
        """ transform the provided es json to geojson.

        :returns: a string representation of the geojson structure
        :rtype: a string
        """
        content = copy.deepcopy(self.content)
        if 'hiddenInformation' in self.content:
            del content['hiddenInformation']
        if ('bbox' in content) and ('coordinates' in content['bbox']):
            coord_table = content['bbox']['coordinates']
            content['bbox'] = []
            content['bbox'].append(coord_table[0][0])
            content['bbox'].append(coord_table[1][1])
            content['bbox'].append(coord_table[1][0])
            content['bbox'].append(coord_table[0][1])
        if ('geometry' in content) and ('type' in content['geometry']):
            if content['geometry']['type'].lower() == "multipolygon":
                content['geometry']['type'] = "MultiPolygon"
            else:
                content['geometry']['type'] = content['geometry']['type'].capitalize()
        if self.validator is not None:
            try:
                self.validator.check(content)
            except:
                OUTPUT_LOGGER.error("Invalid json generated for %s", str(content['id']))
                raise
        return json.dumps(content, indent=4)


class OutputWriter(object):
    """abstract class from which the different writers shall derive."""

    def __init__(self):
        pass

    def output(self, uid, content):
        """ skeleton of the method to write the content.

        :param uid: a uniq reference
        :type uid: a string
        :param content: a string containing the metadata content to save.
        :type content: a string
        """

    def flush(self):
        """ skeleton of the method to flush the pending writes.
        """


class Config(object):
    """class that contains common configuration variables."""

    def __init__(self, config_path=None, initialise=True):
        self.es_servers = None
        self.es_bulk_count = None
        self.es_index = None
        self.es_force_index_documents = None

        # self.kafka_brokers = None
        # self.kafka_secure = None
        # self.kafka_index = None
        # self.kafka_topic = None

        self.file_path = None

        self.collections_index = None
        self.collections_schema = None

        self.products_schema = None
        self.products_statistics = None
        self.products_prod_path = None
        self.products_prod_filter = None
        self.products_meta_path = None
        self.products_meta_key = None
        self.products_meta_parser = None
        self.products_template = None
        self.products_statistics = None

        self.es_source_host = None
        self.es_source_index = None

        self.download_url_prefix = None

        self.s3_endpoint_url = None
        self.s3_unsigned = False

        if initialise:
            filename = os.path.join(resource_path, "configuration.conf")
            configuration = self.read_config_file(filename)
            if configuration is not None:
                self.fix_config(configuration)

        if config_path is not None:
            configuration = self.read_config_file(config_path)
            if configuration is not None:
                self.fix_config(configuration)

    def read_config_file(self, filename):
        """Read a configuration file.

        :param filename: the file name of a configuration file
        :type filename: string
        :returns: a python configuration object with the values present in the file
        :rtype: a python configuration
        """

        configuration = None
        file_object = ResourceReader.get_reader(filename)
        if file_object.isfile():
            try:
                if sys.version_info < (3,):
                    import ConfigParser
                    configuration = ConfigParser.ConfigParser()
                    configuration.readfp(file_object.get_file())
                else:
                    import configparser
                    configuration = configparser.ConfigParser()
                    configuration.read_string(file_object.get_string())
            except (Exception) as error:
                CONFIG_LOGGER.fatal("Configuration file %s can not be parsed (%s).",
                                    str(filename), str(error))
                raise
        else:
            CONFIG_LOGGER.error("Configuration file %s is not found.", str(filename))
        return configuration

    def get_value(self, configuration, node, name, default=None):
        """Read the value associated to a node and a parameter from a configuration.

        :param configuration: a configuration description object
        :type configuration: Config
        :param node: the node to read
        :type node: a string
        :param name: the name of the parameter
        :type name: a string
        :param default: the default value to use if no value is present
        :type default: a string
        :returns: the value associated to the parameter name within the node
        :rtype: a string
        """
        res = None
        try:
            res = configuration.get(node, name)
            if res is None:
                res = default
            elif len(res.strip()) == 0:
                res = None
        except:
            res = default
        return res

    def get_values(self, configuration, node, name, default=None):
        """Read the array associated to a node and a parameter from a configuration.

        :param configuration: a configuration description object
        :type configuration: Config
        :param node: the node to read
        :type node: a string
        :param name: the name of the parameter
        :type name: a string
        :param default: the default value to use if no value is present
        :type default: an array
        :returns: the value associated to the parameter name within the node
        :rtype: an array
        """
        tmp = self.get_value(configuration, node, name, None)
        res = default
        if tmp is not None:
            res = []
            tmp_list = tmp.split(",")
            for val in tmp_list:
                val = val.strip()
                if len(val) > 0:
                    res.append(val.strip())
        return res

    def get_dict(self, configuration, node, name, default=None):
        """Read a dictionary associated to a node and a parameter from a configuration.

        :param configuration: a configuration description object
        :type configuration: Config
        :param node: the node to read
        :type node: a string
        :param name: the name of the parameter
        :type name: a string
        :param default: the default value to use if no value is present
        :type default: an array
        :returns: the value associated to the parameter name within the node
        :rtype: an array
        """
        tmp_list = self.get_values(configuration, node, name, None)
        res = default
        if tmp_list is not None:
            res = {}
            for val in tmp_list:
                splitted2 = val.split(":")
                if len(splitted2) > 1:
                    res[splitted2[0].strip()] = splitted2[1].strip()
        return res

    def fix_config(self, configuration):
        """updates the properties of the object according the provided configuration.

        :param configuration: a configuration description object
        :type configuration: Config
        """
        section_Es = "elasticsearch"
        # section_Kafka = "kafka"
        section_file = "file"
        section_collections = "collections"
        section_products = "products"
        section_download = "download"
        # section_s3 = "s3"
        self.es_servers = self.get_values(configuration,
                                          section_Es,
                                          "servers",
                                          self.es_servers)
        self.es_index = self.get_value(configuration,
                                       section_Es,
                                       "index",
                                       self.es_index)
        tmp = self.get_value(configuration,
                             section_Es,
                             "bulk_count")
        if tmp is not None:
            self.es_bulk_count = int(tmp)
        tmp = self.get_value(configuration, section_Es, "force_index_documents")
        self.es_force_index_documents = None if ((tmp is None) or tmp == "0" or tmp.lower() == "false") else tmp

        # self.kafka_brokers = self.get_values(configuration,
        #                                      section_Kafka,
        #                                      "brokers",
        #                                      self.kafka_brokers)

        # tmp = self.get_value(configuration, section_Kafka, "secure")
        # self.kafka_secure = None if ((tmp is None) or tmp == "0" or tmp.lower() == "false") else tmp
        # self.kafka_index = self.get_value(configuration,
        #                                   section_Kafka,
        #                                   "index",
        #                                   self.kafka_index)
        # self.kafka_topic = self.get_value(configuration, section_Kafka, "topic", self.kafka_topic)

        self.file_path = self.get_value(configuration, section_file, "path", self.file_path)

        self.collections_index = self.get_value(configuration,
                                                section_collections,
                                                "index",
                                                self.collections_index)

        self.collections_schema = self.get_value(configuration,
                                                 section_collections,
                                                 "schema",
                                                 self.collections_schema)

        self.products_schema = self.get_value(configuration,
                                              section_products,
                                              "schema",
                                              self.products_schema)
        tmp = self.get_value(configuration, section_products, "statistics")
        self.products_statistics = None if ((tmp is None) or tmp == "0" or tmp.lower() == "false") else tmp
        self.products_prod_path = self.get_value(configuration,
                                                 section_products,
                                                 "prod_path",
                                                 self.products_prod_path)
        self.products_prod_filter = self.get_value(configuration,
                                                   section_products,
                                                   "prod_filter",
                                                   self.products_prod_filter)
        self.products_meta_path = self.get_value(configuration,
                                                 section_products,
                                                 "meta_path",
                                                 self.products_meta_path)
        self.products_meta_key = self.get_value(configuration,
                                                section_products,
                                                "meta_key",
                                                self.products_meta_key)
        self.products_meta_parser = self.get_value(configuration,
                                                   section_products,
                                                   "meta_parser",
                                                   self.products_meta_parser)
        self.products_template = self.get_value(configuration,
                                                section_products,
                                                "template",
                                                self.products_template)
        self.es_source_host = self.get_values(configuration,
                                              section_products,
                                              "es_source_host",
                                              self.es_source_host)
        self.es_source_index = self.get_value(configuration,
                                              section_products,
                                              "es_source_index",
                                              self.es_source_index)
        self.download_url_prefix = self.get_value(configuration,
                                                  section_download,
                                                  "url_prefix",
                                                  self.download_url_prefix)

        # self.s3_endpoint_url = self.get_value(configuration, section_s3, "endpoint")
        # self.s3_unsigned = self.get_value(configuration, section_s3, "unsigned", self.s3_unsigned)
