import glob
import os
import time
import logging
from typing import Iterator

LOGGER = logging.getLogger(__name__)


class StorageInput:

    def __init__(self):
        pass

    def glob(self, path: str) -> Iterator[str]:
        """
        Get the paths matching the provided glob pattern.

        :param path: glob path pattern
        :returns: iterator of paths
        """
        raise NotImplementedError

    def list_files_recursively(self, path: str) -> Iterator[str]:
        """
        List all files in the given directory path recursively. If the path is a file, only that file will be returned.

        :param path: path of file or directory to list
        :returns: iterator of file paths
        """
        raise NotImplementedError

    def get_filesize(self, path: str) -> int:
        """
        Get the file size, in bytes, of the given path.

        :param path: path of the file
        :returns: file size in bytes
        """
        raise NotImplementedError

    def get_time_modified(self, path) -> time.struct_time:
        """
        Get the modification timestamp of the given file path.

        :param path: path of the file
        :returns: modification date of file
        """
        raise NotImplementedError

    def open_file_lines(self, path: str):
        raise NotImplementedError


class FileSystemInput(StorageInput):

    def glob(self, path: str) -> Iterator[str]:
        return glob.iglob(path)

    def list_files_recursively(self, path: str) -> Iterator[str]:
        if os.path.isfile(path):
            yield path
        elif os.path.isdir(path):
            for filename in glob.iglob(os.path.join(path, "**"), recursive=True):
                if os.path.isfile(filename):
                    yield filename

    def get_filesize(self, path: str) -> int:
        return os.path.getsize(path)

    def get_time_modified(self, path) -> time.struct_time:
        return time.localtime(os.path.getmtime(path))

    def open_file_lines(self, path: str):
        return open(path, 'r')
